package betterquesting.questing.party;

import betterquesting.api.enums.EnumPartyStatus;
import betterquesting.api.properties.IPropertyContainer;
import betterquesting.api.properties.IPropertyType;
import betterquesting.api.properties.NativeProps;
import betterquesting.api.questing.party.IParty;
import betterquesting.core.BetterQuesting;
import betterquesting.storage.PropertyContainer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import javax.annotation.Nonnull;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;

/* loaded from: betterquesting_questing_party.jar:betterquesting/questing/party/PartyInstance.class */
public class PartyInstance implements IParty {
    private final HashMap<UUID, EnumPartyStatus> members = new HashMap<>();
    private List<UUID> memCache = null;
    private final PropertyContainer pInfo = new PropertyContainer();

    public PartyInstance() {
        setupProps();
    }

    private void setupProps() {
        setupValue(NativeProps.NAME, "New Party");
    }

    /* JADX WARN: Multi-variable type inference failed */
    private <T> void setupValue(IPropertyType<T> prop) {
        setupValue(prop, prop.getDefault());
    }

    private <T> void setupValue(IPropertyType<T> prop, T def) {
        this.pInfo.setProperty(prop, this.pInfo.getProperty(prop, def));
    }

    private void refreshCache() {
        this.memCache = Collections.unmodifiableList(new ArrayList(this.members.keySet()));
    }

    public IPropertyContainer getProperties() {
        return this.pInfo;
    }

    public void kickUser(@Nonnull UUID uuid) {
        if (this.members.containsKey(uuid)) {
            EnumPartyStatus old = this.members.get(uuid);
            this.members.remove(uuid);
            if (old == EnumPartyStatus.OWNER && this.members.size() > 0) {
                hostMigrate();
            }
            refreshCache();
        }
    }

    public void setStatus(@Nonnull UUID uuid, @Nonnull EnumPartyStatus priv) {
        EnumPartyStatus old = this.members.get(uuid);
        if (old == priv) {
            return;
        }
        this.members.put(uuid, priv);
        if (priv == EnumPartyStatus.OWNER) {
            for (UUID mem : getMembers()) {
                if (mem != uuid && this.members.get(mem) == EnumPartyStatus.OWNER) {
                    this.members.put(mem, EnumPartyStatus.ADMIN);
                }
            }
        } else if (old == EnumPartyStatus.OWNER) {
            UUID migrate = null;
            Iterator<UUID> it = getMembers().iterator();
            while (true) {
                if (!it.hasNext()) {
                    break;
                }
                UUID mem2 = it.next();
                if (mem2 != uuid) {
                    if (this.members.get(mem2) == EnumPartyStatus.ADMIN) {
                        migrate = mem2;
                        break;
                    } else if (migrate == null) {
                        migrate = mem2;
                    }
                }
            }
            if (migrate == null) {
                this.members.put(uuid, old);
                return;
            }
            this.members.put(migrate, EnumPartyStatus.OWNER);
        }
        refreshCache();
    }

    public EnumPartyStatus getStatus(@Nonnull UUID uuid) {
        return this.members.get(uuid);
    }

    public List<UUID> getMembers() {
        if (this.memCache == null) {
            refreshCache();
        }
        return this.memCache;
    }

    private void hostMigrate() {
        for (UUID uuid : this.members.keySet()) {
            if (this.members.get(uuid) == EnumPartyStatus.OWNER) {
                return;
            }
        }
        UUID migrate = null;
        for (UUID mem : this.members.keySet()) {
            EnumPartyStatus status = this.members.get(mem);
            if (status == EnumPartyStatus.ADMIN || status == EnumPartyStatus.OWNER) {
                migrate = mem;
                break;
            } else if (migrate == null) {
                migrate = mem;
            }
        }
        if (migrate != null) {
            this.members.put(migrate, EnumPartyStatus.OWNER);
        } else {
            BetterQuesting.logger.error("Failed to find suitable host to migrate party " + ((String) this.pInfo.getProperty(NativeProps.NAME)));
        }
    }

    public NBTTagCompound writeToNBT(NBTTagCompound json) {
        NBTTagList memJson = new NBTTagList();
        for (Map.Entry<UUID, EnumPartyStatus> mem : this.members.entrySet()) {
            NBTTagCompound jm = new NBTTagCompound();
            jm.func_74778_a("uuid", mem.getKey().toString());
            jm.func_74778_a("status", mem.getValue().toString());
            memJson.func_74742_a(jm);
        }
        json.func_74782_a("members", memJson);
        json.func_74782_a("properties", this.pInfo.writeToNBT(new NBTTagCompound()));
        return json;
    }

    public void readFromNBT(NBTTagCompound jObj) {
        if (jObj.func_150297_b("properties", 10)) {
            this.pInfo.readFromNBT(jObj.func_74775_l("properties"));
        } else {
            this.pInfo.readFromNBT(new NBTTagCompound());
            this.pInfo.setProperty(NativeProps.NAME, jObj.func_74779_i("name"));
        }
        this.members.clear();
        NBTTagList memList = jObj.func_150295_c("members", 10);
        for (int i = 0; i < memList.func_74745_c(); i++) {
            try {
                NBTTagCompound jMem = memList.func_150305_b(i);
                if (jMem.func_150297_b("uuid", 8) && jMem.func_74764_b("status")) {
                    UUID uuid = UUID.fromString(jMem.func_74779_i("uuid"));
                    EnumPartyStatus priv = EnumPartyStatus.valueOf(jMem.func_74779_i("status"));
                    this.members.put(uuid, priv);
                }
            } catch (Exception e) {
            }
        }
        refreshCache();
        setupProps();
    }

    public NBTTagCompound writeProperties(NBTTagCompound nbt) {
        return this.pInfo.writeToNBT(nbt);
    }

    public void readProperties(NBTTagCompound nbt) {
        this.pInfo.readFromNBT(nbt);
    }
}
